import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_svg/flutter_svg.dart';
import 'package:syncrow_web/pages/auth/bloc/auth_bloc.dart';
import 'package:syncrow_web/pages/auth/bloc/auth_event.dart';
import 'package:syncrow_web/pages/auth/bloc/auth_state.dart';
import 'package:syncrow_web/pages/auth/model/region_model.dart';
import 'package:syncrow_web/pages/common/default_button.dart';
import 'package:syncrow_web/pages/common/first_layer.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/constants/assets.dart';
import 'package:syncrow_web/utils/style.dart';

class ForgetPasswordWebPage extends StatelessWidget {
  const ForgetPasswordWebPage({super.key});
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: BlocProvider(
        create: (context) => AuthBloc()..add(RegionInitialEvent()),
        child: BlocConsumer<AuthBloc, AuthState>(
          listener: (context, state) {
            if (state is SuccessForgetState) {
              ScaffoldMessenger.of(context).showSnackBar(
                const SnackBar(
                  content: Text('Change Password Successfully '),
                ),
              );
              Navigator.of(context).pop();
            } else if (state is FailureForgetState) {
              ScaffoldMessenger.of(context).showSnackBar(
                SnackBar(
                  content: Text(state.error),
                ),
              );
            }
          },
          builder: (context, state) {
              return _buildForm(context, state);
          },
        ),
      ),
    );
  }

  Widget _buildForm(BuildContext context, AuthState state) {
    late ScrollController _scrollController;
    _scrollController = ScrollController();
    void _scrollToCenter() {
      final double middlePosition =
          _scrollController.position.maxScrollExtent / 2;
      _scrollController.animateTo(
        middlePosition,
        duration: const Duration(seconds: 1),
        curve: Curves.easeInOut,
      );
    }

    WidgetsBinding.instance.addPostFrameCallback((_) {
      _scrollToCenter();
    });
    final forgetBloc = BlocProvider.of<AuthBloc>(context);
    Size size = MediaQuery.of(context).size;
    return FirstLayer(
        second: Center(
      child: Stack(
        children: [
          if (state is AuthLoading)
         const Center(child: CircularProgressIndicator()),
          ListView(
            shrinkWrap: true,
            controller: _scrollController,
            children: [
              Container(
                padding: EdgeInsets.all(size.width * 0.02),
                margin: EdgeInsets.all(size.width * 0.09),
                decoration: BoxDecoration(
                  color: Colors.black.withOpacity(0.3),
                  borderRadius: const BorderRadius.all(Radius.circular(20)),
                ),
                child: Center(
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      const Spacer(),
                      Expanded(
                        flex: 3,
                        child: SvgPicture.asset(
                          Assets.loginLogo,
                        ),
                      ),
                      const Spacer(),
                      Expanded(
                        flex: 3,
                        child: Container(
                          decoration: BoxDecoration(
                            color: Colors.white.withOpacity(0.1),
                            borderRadius:
                                const BorderRadius.all(Radius.circular(30)),
                            border: Border.all(
                                color: ColorsManager.graysColor.withOpacity(0.2)),
                          ),
                          child: Form(
                            key: forgetBloc.forgetFormKey,
                            child: Padding(
                              padding: EdgeInsets.symmetric(
                                  horizontal: size.width * 0.02,
                                  vertical: size.width * 0.003),
                              child: Column(
                                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: <Widget>[
                                  const SizedBox(height: 10),
                                  const Text(
                                    'Forget Password',
                                    style: TextStyle(
                                        color: Colors.white,
                                        fontSize: 24,
                                        fontWeight: FontWeight.bold),
                                  ),
                                  const SizedBox(height: 10),
                                  Text(
                                    'Please fill in your account information to\nretrieve your password',
                                    style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                        fontSize: 14,
                                        fontWeight: FontWeight.w400),
                                  ),
                                  const SizedBox(height: 10),
                                  Column(
                                    crossAxisAlignment: CrossAxisAlignment.start,
                                    mainAxisAlignment: MainAxisAlignment.start,
                                    children: [
                                      Text(
                                        "Country/Region",
                                        style: Theme.of(context).textTheme.bodySmall!.copyWith(
                                            fontSize: 14,
                                            fontWeight: FontWeight.w400),

                                      ),
                                      const SizedBox(height: 10),
                                      SizedBox(
                                        child: DropdownButtonFormField<String>(
                                          validator: forgetBloc.validateRegion,
                                          icon: const Icon(
                                            Icons.keyboard_arrow_down_outlined,
                                          ),
                                          decoration: textBoxDecoration()!.copyWith(
                                            hintText: null,
                                          ),
                                          hint: SizedBox(
                                            width: size.width * 0.12,
                                            child: const Align(
                                              alignment: Alignment.centerLeft,
                                              child: Text(
                                                'Select your region/country',
                                                textAlign: TextAlign.center,
                                                overflow: TextOverflow.ellipsis,
                                              ),
                                            ),
                                          ),
                                          isDense: true,
                                          style:
                                              const TextStyle(color: Colors.black),
                                          items: forgetBloc.regionList!
                                              .map((RegionModel region) {
                                            return DropdownMenuItem<String>(
                                              value: region.id,
                                              child: SizedBox(
                                                  width: size.width*0.06,

                                                  child: Text(region.name)),
                                            );
                                          }).toList(),
                                          onChanged: (String? value) {
                                            forgetBloc.add(SelectRegionEvent(
                                              val: value!,
                                            ));
                                          },
                                        ),
                                      )
                                    ],
                                  ),
                                  const SizedBox(height: 20),
                                  Column(
                                    crossAxisAlignment: CrossAxisAlignment.start,
                                    mainAxisAlignment: MainAxisAlignment.start,
                                    children: [
                                      Text(
                                        "Account",
                                        style: Theme.of(context).textTheme.bodySmall!.copyWith(fontSize: 14,fontWeight: FontWeight.w400),

                                      ),
                                      const SizedBox(height: 10),
                                      SizedBox(
                                        child: TextFormField(
                                          validator: forgetBloc.validateEmail,
                                          controller:
                                              forgetBloc.forgetEmailController,
                                          decoration: textBoxDecoration()!.copyWith(
                                              hintText: 'Enter your email'),
                                          style:
                                              const TextStyle(color: Colors.black),
                                        ),
                                      ),
                                    ],
                                  ),
                                  const SizedBox(height: 20.0),
                                  Column(
                                    crossAxisAlignment: CrossAxisAlignment.start,
                                    mainAxisAlignment: MainAxisAlignment.start,
                                    children: [
                                      Text(
                                        "One Time Password",
                                        style: Theme.of(context).textTheme.bodySmall!.copyWith(fontSize: 14,fontWeight: FontWeight.w400),
                                      ),
                                      const SizedBox(height: 10),
                                      SizedBox(
                                        child: TextFormField(
                                          validator: forgetBloc.validateCode,
                                          keyboardType:
                                              TextInputType.visiblePassword,
                                          controller: forgetBloc.forgetOtp,
                                          decoration: textBoxDecoration()!.copyWith(
                                            hintText: 'Enter Code',
                                            suffixIcon: SizedBox(
                                              width: 100,
                                              child: Center(
                                                child: InkWell(
                                                  onTap:state is TimerState && !state.isButtonEnabled && state.remainingTime!=1?null: () {
                                                    forgetBloc.add(StartTimerEvent());
                                                  },
                                                  child: Text(
                                                    'Get Code ${state is TimerState && !state.isButtonEnabled && state.remainingTime!=1? "(${forgetBloc.formattedTime(state.remainingTime)}) " : ""}',
                                                    style: TextStyle(
                                                      color: state is TimerState &&
                                                              !state.isButtonEnabled
                                                          ? Colors.grey
                                                          : ColorsManager.btnColor,
                                                    ),
                                                  ),
                                                ),
                                              ),
                                            ),
                                          ),
                                          style:
                                              const TextStyle(color: Colors.black),
                                        ),
                                      ),
                                      if (forgetBloc.forgetValidate != '') // Check if there is a validation message
                                        Padding(
                                          padding: const EdgeInsets.only(top: 8.0),
                                          child: Text(
                                            forgetBloc.forgetValidate,
                                            style: const TextStyle(
                                              color: ColorsManager.red,
                                              fontSize: 10,
                                              fontWeight: FontWeight.w700
                                            ),
                                          ),
                                        ),
                                    ],
                                  ),
                                  const SizedBox(height: 20.0),
                                  Column(
                                    crossAxisAlignment: CrossAxisAlignment.start,
                                    mainAxisAlignment: MainAxisAlignment.start,
                                    children: [
                                      Text(
                                        "Password",
                                        style: Theme.of(context).textTheme.bodySmall!.copyWith(fontSize: 14,fontWeight: FontWeight.w400),
                                      ),
                                      const SizedBox(height: 10),
                                      SizedBox(
                                        child: TextFormField(
                                          validator: forgetBloc.passwordValidator,
                                          keyboardType: TextInputType.visiblePassword,
                                          controller: forgetBloc.forgetPasswordController,
                                          decoration: textBoxDecoration()!.copyWith(
                                            hintText: 'At least 8 characters',
                                          ),
                                          style: const TextStyle(color: Colors.black),
                                        ),
                                      ),
                                    ],
                                  ),
                                  const SizedBox(
                                    height: 10,
                                  ),
                                  const SizedBox(height: 20.0),
                                  Row(
                                    crossAxisAlignment: CrossAxisAlignment.center,
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    children: [
                                      SizedBox(
                                        width: size.width * 0.2,
                                        child: DefaultButton(
                                          backgroundColor: ColorsManager.btnColor,
                                          child: const Text('Submit'),
                                          onPressed: () {
                                            if (forgetBloc.forgetFormKey.currentState!.validate()) {
                                              forgetBloc.add(ChangePasswordEvent());
                                            }
                                          },
                                        ),
                                      ),
                                    ],
                                  ),
                                  const SizedBox(height: 10.0),
                                  SizedBox(
                                    child: Text(
                                      forgetBloc.validate,
                                      style: const TextStyle(
                                          fontWeight: FontWeight.w700,
                                          color: ColorsManager.red),
                                    ),
                                  ),
                                  SizedBox(
                                    height: 10,
                                  ),
                                  SizedBox(
                                    width: size.width * 0.2,
                                    child: Row(
                                      mainAxisAlignment: MainAxisAlignment.center,
                                      crossAxisAlignment: CrossAxisAlignment.center,
                                      children: [
                                        const Flexible(
                                            child: Text(
                                          "Do you have an account? ",
                                          style: TextStyle(color: Colors.white),
                                        )),
                                        InkWell(
                                          onTap: () {
                                            forgetBloc.add(StopTimerEvent());
                                            Navigator.pop(context);
                                          },
                                          child: const Flexible(
                                              child: Text(
                                            "Sign in",
                                          )),
                                        ),
                                      ],
                                    ),
                                  ),
                                  const SizedBox(height: 15.0),
                                ],
                              ),
                            ),
                          ),
                        ),
                      ),
                      const Spacer(),
                    ],
                  ),
                ),
              ),
            ],
          ),
        ],
      ),
    ));
  }
}
